"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertSOQueriesToPack = exports.convertPackQueriesToSO = void 0;

var _lodash = require("lodash");

var _constants = require("../../../common/constants");

var _remove_multilines = require("../../../common/utils/build_query/remove_multilines");

var _utils = require("../utils");

/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */
// @ts-expect-error update types
const convertPackQueriesToSO = queries => (0, _lodash.reduce)(queries, (acc, value, key) => {
  const ecsMapping = value.ecs_mapping && (0, _utils.convertECSMappingToArray)(value.ecs_mapping);
  acc.push({
    id: key,
    ...(0, _lodash.pick)(value, ['name', 'query', 'interval', 'platform', 'version']),
    ...(value.snapshot !== undefined ? {
      snapshot: value.snapshot
    } : {}),
    ...(value.removed !== undefined ? {
      removed: value.removed
    } : {}),
    ...(ecsMapping ? {
      ecs_mapping: ecsMapping
    } : {})
  });
  return acc;
}, []);

exports.convertPackQueriesToSO = convertPackQueriesToSO;

const convertSOQueriesToPack = ( // @ts-expect-error update types
queries, options) => (0, _lodash.reduce)(queries, // eslint-disable-next-line @typescript-eslint/naming-convention
(acc, {
  id: queryId,
  ecs_mapping,
  query,
  platform,
  removed,
  snapshot,
  ...rest
}, key) => {
  const resultType = !snapshot ? {
    removed,
    snapshot
  } : {};
  const index = queryId ? queryId : key;
  acc[index] = { ...rest,
    query: options !== null && options !== void 0 && options.removeMultiLines ? (0, _remove_multilines.removeMultilines)(query) : query,
    ...(!(0, _lodash.isEmpty)(ecs_mapping) ? (0, _lodash.isArray)(ecs_mapping) ? {
      ecs_mapping: (0, _utils.convertECSMappingToObject)(ecs_mapping)
    } : {
      ecs_mapping
    } : {}),
    ...(platform === _constants.DEFAULT_PLATFORM || platform === undefined ? {} : {
      platform
    }),
    ...(options !== null && options !== void 0 && options.removeResultType ? resultType : { ...(snapshot ? {
        snapshot
      } : {}),
      ...(removed ? {
        removed
      } : {})
    })
  };
  return acc;
}, // eslint-disable-next-line @typescript-eslint/no-explicit-any
{});

exports.convertSOQueriesToPack = convertSOQueriesToPack;